<?php
namespace Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class CMP07400ControleDeAlteracaoSolicitacaoDeCompras {
    
    protected $environment;
    protected $dataSourceManager;
    protected $newCode;
    protected $connection;
    public $verifSoli = null;
    public $centCmp = null;
    public $verifItemSoli = null;
    public $parCmpSup = null;
    
    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager, $dataSourceManager, NewCode $newCode, $connection) {
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode           = $newCode;
        $this->connection        = $connection;
    }
    
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        $sql   = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmDelItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLAN" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTPROGENTR" => $DTPROGENTR
        );
        $sql   = "DELETE FROM ITEMSOLI WHERE CDFILSOLI = :P_CDFILSOLI AND CDFILLAN = :P_CDFILLAN AND NRSOLICMP = :P_NRSOLICMP AND DTPROGENTR = :P_DTPROGENTR AND CDPRODUTO = :P_CDPRODUTO";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmInsertMovopSol($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $DTOPERSOL, $CDOPERADOR, $QTANTOPER, $QTDPOPER, $IDOPERAMOVS) {
        $params = array(
            "P_CDFILSOLMOV" => $CDFILSOLI,
            "P_CDFILLANMOV" => $CDFILLAN,
            "P_NRSOLIMOV" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTENTROPER" => $DTPROGENTR,
            "P_DTOPERSOL" => $DTOPERSOL,
            "P_CDOPERADOR" => $CDOPERADOR,
            "P_QTANTOPER" => $QTANTOPER,
            "P_QTDEPOPER" => $QTDPOPER,
            "P_IDOPERAMOVS" => $IDOPERAMOVS
        );
        $sql   = "INSERT INTO MOVOPSOL(DTOPERSOL, CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, CDPRODUTO, DTENTROPER, QTANTOPER, QTDEPOPER, IDOPERAMOVS, CDOPERADOR) VALUES (:P_DTOPERSOL, :P_CDFILSOLMOV, :P_CDFILLANMOV, :P_NRSOLIMOV, :P_CDPRODUTO, :P_DTENTROPER, :P_QTANTOPER, :P_QTDEPOPER, :P_IDOPERAMOVS, :P_CDOPERADOR)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmDelItemSoliDel($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLAN" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTPROGENTR" => $DTPROGENTR
        );
        $sql   = "DELETE FROM ITEMSOLIDEL WHERE CDFILSOLI = :P_CDFILSOLI AND CDFILLAN = :P_CDFILLAN AND NRSOLICMP = :P_NRSOLICMP AND CDPRODUTO = :P_CDPRODUTO AND DTPROGENTR = :P_DTPROGENTR";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function adqVerifSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLANCA" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP
        );
        $sql   = "SELECT 1 FROM SOLICITA WHERE CDFILSOLI = :P_CDFILSOLI AND CDFILLANCA = :P_CDFILLANCA AND NRSOLICMP = :P_NRSOLICMP";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $this->verifSoli = $query->fetchAll();
    }
    
    public function adqCentCmp($CDFILSOLI) {
        $params = array(
            "P_CDFILIAL" => $CDFILSOLI
        );
        
        $sql   = "SELECT NVL(FIL.CDFILCENTCMP, FIL.CDFILIAL) AS CDFILIAL, TRUNC(SYSDATE) AS DATA FROM PARASUPRI PAR, FILIAL FIL WHERE FIL.CDFILIAL = :P_CDFILIAL AND PAR.CDFILIAL = NVL(FIL.CDFILCENTCMP, FIL.CDFILIAL)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $this->centCmp = $query->fetchAll();
    }
    
    public function acmInsSolicita($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $DTSOLCMP, $CDFILCENT) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLANCA" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_DTSOLCMP" => $DTSOLCMP,
            "P_CDFILCENT" => $CDFILCENT,
            "P_IDTIPLAN" => 'M'
        );
        $sql   = "INSERT INTO SOLICITA(CDFILLANCA, CDFILSOLI, NRSOLICMP, DTSOLCMP, IDTIPLAN, CDFILCENT) VALUES (:P_CDFILLANCA, :P_CDFILSOLI, :P_NRSOLICMP, :P_DTSOLCMP, :P_IDTIPLAN, :P_CDFILCENT)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function adqVerifItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLAN" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTPROGENTR" => $DTPROGENTR
        );
        $sql   = "SELECT '1' FROM ITEMSOLI WHERE CDFILSOLI = :P_CDFILSOLI AND CDFILLAN = :P_CDFILLAN AND NRSOLICMP = :P_NRSOLICMP AND CDPRODUTO = :P_CDPRODUTO AND DTPROGENTR = :P_DTPROGENTR";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $this->verifItemSoli = $query->fetchAll();
    }
    
    public function acmInsItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $QTSOLIPROD, $DTUTILIZA, $IDSOLIAPROV, $DTHRAPROVSOL, $CDOPERADOR) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLAN" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTPROGENTR" => $DTPROGENTR,
            "P_QTSOLIPROD" => $QTSOLIPROD,
            "P_DTUTILIZA" => $DTUTILIZA,
            "P_QTVOLUITEMSO" => 0,
            "P_IDSOLIAPROV" => $IDSOLIAPROV,
            "P_DTHRAPROVSOL" => $DTHRAPROVSOL,
            "P_CDOPERADOR" => $CDOPERADOR
        );
        $sql   = "INSERT INTO ITEMSOLI(CDFILSOLI, CDFILLAN, NRSOLICMP, CDPRODUTO, DTPROGENTR, QTSOLIPROD, DTUTILIZA, IDSOLIAPROV, QTVOLUITEMSO, DTHRAPROVSOL, CDOPERAPROV) VALUES (:P_CDFILSOLI, :P_CDFILLAN, :P_NRSOLICMP, :P_CDPRODUTO, :P_DTPROGENTR, :P_QTSOLIPROD, :P_DTUTILIZA, :P_IDSOLIAPROV, :P_QTVOLUITEMSO, :P_DTHRAPROVSOL, :P_CDOPERADOR)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function adqParCmpSup($CDFILSOLI) {
        $params = array(
            "P_CDFILIAL" => $CDFILSOLI
        );
        $sql   = "SELECT IDUTAPROVSOL FROM PARCMPSUP WHERE CDFILIAL = :P_CDFILIAL";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $this->parCmpSup = $query->fetchAll();
    }
    
    public function acmUpdItemSoliNull($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $QTSOLIPROD) {
        $params = array(
            "P_CDFILSOLI" => $CDFILSOLI,
            "P_CDFILLAN" => $CDFILLAN,
            "P_NRSOLICMP" => $NRSOLICMP,
            "P_CDPRODUTO" => $CDPRODUTO,
            "P_DTPROGENTR" => $DTPROGENTR,
            "P_QTSOLIPROD" => $QTSOLIPROD
        );
        $sql   = "UPDATE ITEMSOLI SET QTSOLIORIG = NULL, QTSOLIPROD = :P_QTSOLIPROD WHERE CDFILSOLI = :P_CDFILSOLI AND CDFILLAN = :P_CDFILLAN AND NRSOLICMP = :P_NRSOLICMP AND CDPRODUTO = :P_CDPRODUTO AND DTPROGENTR = :P_DTPROGENTR";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmTodasFiliais($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        $sql    = "INSERT INTO LISTA(NRLISTA, CDLISTA) (SELECT :NRLISTA, CDFILIAL FROM FILIAL WHERE EXISTS(SELECT 1 FROM SOLICITA WHERE CDFILSOLI = CDFILIAL))";
        $query  = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    
    public function acmTodasFilAgrup($nrlista, $cdgrupfili) {
        $params = array(
            "NRLISTA" => $nrlista,
            "P_CDGRUPFILI" => $cdgrupfili
        );
        $sql   = "INSERT INTO LISTA(NRLISTA, CDLISTA) (SELECT :NRLISTA, CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDGRUPFILI) ";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    
    public function setgrid(DTO\Request\Row $request, DTO\Response $response) {
        $row     = $request->getRow();
        $NRLISTA = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        
        if ($row['CDFILIAL'] == 'T') {
            if ($row['NMGRUPFILI'] === null) {
                $this->acmTodasFiliais($NRLISTA);
            } else {
                $this->acmTodasFilAgrup($NRLISTA, $row['CDGRUPFILI']);
            }
        } else {
            $filiais = array();
            $filiais = $row['CDFILIAL'];
            $filiais = explode("_", $row['CDFILIAL']);
            foreach ($filiais as $filial) {
                $this->insertLista($NRLISTA, $filial);
            }
        }
        $value = array(
            $NRLISTA
        );
        $response->addDataSet(new DataSet("NrLista", $value));
    }
    
    public function confirmaAlteracao(DTO\Request\Row $request, DTO\Response $response) {
        $rows = $request->getRow();
        
        $DTOPERSOL  = $rows[0]['SYSDATE'];
        $CDOPERADOR = $rows[0]['CDOPERADOR'];
        
        foreach ($rows as $row) {
            $CDFILSOLI  = $row['CDFILSOLI'];
            $CDFILLAN   = $row['CDFILLAN'];
            $NRSOLICMP  = $row['NRSOLICMP'];
            $CDPRODUTO  = $row['CDPRODUTO'];
            $DTPROGENTR = $row['DTPROGENTR'];
            $QTSOLIPROD = $row['QTSOLIPROD'];
            $QTSOLIORIG = $row['QTSOLIORIG'];
            
            if ($QTSOLIORIG == 0) {
                $this->acmDelItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR);
                $this->acmInsertMovopSol($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $DTOPERSOL, $CDOPERADOR, $QTSOLIPROD, 0, 'E');
            } else if ($QTSOLIPROD == 0 && $QTSOLIORIG > 0) {
                $this->acmDelItemSoliDel($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR);
                $this->adqVerifSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP);
     
                if (empty($this->verifSoli)) {
                    $this->adqCentCmp($CDFILSOLI);
                    
                    if (empty($this->centCmp)) {
                        $this->acmInsSolicita($CDFILSOLI, $CDFILLAN, $NRSOLICMP, null, null);
                    } else {
                        $this->acmInsSolicita($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $this->centCmp['0']['DATA'], $this->centCmp['0']['CDFILIAL']);
                    }
                }
                
                $this->adqVerifItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR);
                
                if (empty($this->verifItemSoli)) {
                    $this->adqParCmpSup($CDFILSOLI);
                    
                    if ($this->parCmpSup['0']['IDUTAPROVSOL'] == 'S') {
                        $IDSOLIAPROV = 'N';
                    } else {
                        $IDSOLIAPROV = 'S';
                    }
                    
                    $this->acmInsItemSoli($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $QTSOLIPROD, $DTPROGENTR, $IDSOLIAPROV, $DTOPERSOL, $CDOPERADOR);
                    $this->acmInsertMovopSol($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $DTOPERSOL, $CDOPERADOR, 0, $QTSOLIPROD, 'I');
                    
                } else {
                    $this->acmUpdItemSoliNull($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $QTSOLIPROD);
                    $this->acmInsertMovopSol($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $DTOPERSOL, $CDOPERADOR, 0, $QTSOLIORIG, 'A');
                }
                
            } else if ($QTSOLIPROD > 0 && $QTSOLIORIG > 0) {
                $this->acmUpdItemSoliNull($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $QTSOLIORIG);
                $this->acmInsertMovopSol($CDFILSOLI, $CDFILLAN, $NRSOLICMP, $CDPRODUTO, $DTPROGENTR, $DTOPERSOL, $CDOPERADOR, $QTSOLIPROD, $QTSOLIORIG, 'A');
            }
        }
    }
} 